require "import"
import "android.app.*"
import "android.content.*"
import "android.os.*"
import "android.view.*"
import "android.widget.*"
import "java.io.File"
import "java.io.FileWriter"
import "java.io.FileReader"
import "java.io.BufferedReader"
import "java.io.BufferedWriter"
import "java.util.ArrayList"
import "android.media.MediaRecorder"

-- Constants
local STORAGE_PATH = "/storage/emulated/0/解说/Tools/notetaking-tool_pro/notes.txt"
local FAVORITES_PATH = "/storage/emulated/0/解说/Tools/notetaking-tool_pro/favorites.txt"
local RECORDING_FOLDER_PATH = "/storage/emulated/0/解说/Tools/notetaking-tool_pro/recordings/"

-- UI Elements
local activity = activity
local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)

-- Heading
local headingTextView = TextView(activity)
headingTextView.setText("notetaking-tool_pro")
headingTextView.setTextSize(20)
headingTextView.setGravity(Gravity.CENTER)

-- Search button
local searchButton = Button(activity)
searchButton.setText("Search")

-- Notes button
local notesButton = Button(activity)
notesButton.setText("Notes")

-- Favorites button
local favoritesButton = Button(activity)
favoritesButton.setText("Favorites")

-- Add Note button
local addNoteButton = Button(activity)
addNoteButton.setText("Add Note")

-- New Recording button
local newRecordingButton = Button(activity)
newRecordingButton.setText("New Recording")

-- Developer Info button
local developerInfoButton = Button(activity)
developerInfoButton.setText("Developer Info")

-- Notes ListView
local notesListView = ListView(activity)
local notesList = ArrayList()
local notesAdapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, notesList)
notesListView.setAdapter(notesAdapter)

-- Load existing notes
local function loadNotes()
  local file = File(STORAGE_PATH)

  if file.exists() then
    local reader = BufferedReader(FileReader(file))
    local line

    repeat
      line = reader.readLine()
      if line then
        notesList.add(line)
      end
    until not line

    reader.close()
    notesAdapter.notifyDataSetChanged()
  end
end

-- Save a new note to file
local function saveNote(note)
  local file = File(STORAGE_PATH)
  if not file.exists() then
    file.getParentFile().mkdirs()
    file.createNewFile()
  end

  local fileWriter = FileWriter(file, true)
  local bufferedWriter = BufferedWriter(fileWriter)

  bufferedWriter.write(note)
  bufferedWriter.newLine()

  bufferedWriter.close()
  fileWriter.close()

  notesList.add(note)
  notesAdapter.notifyDataSetChanged()
end

-- Remove a note from file
local function removeNote(position)
  local file = File(STORAGE_PATH)
  local tempFile = File("/storage/emulated/0/解说/Tools/notetaking-tool_pro/temp_notes.txt")

  local scanner = BufferedReader(FileReader(file))
  local writer = BufferedWriter(FileWriter(tempFile))

  local currentLine = 0
  local line

  repeat
    line = scanner.readLine()
    if line then
      if currentLine ~= position then
        writer.write(line)
        writer.newLine()
      end
    end
    currentLine = currentLine + 1
  until not line

  scanner.close()
  writer.close()

  -- Delete the original file
  file.delete()

  -- Rename the temporary file to the original file name
  tempFile.renameTo(file)

  notesList.remove(position)
  notesAdapter.notifyDataSetChanged()
end

-- Save a note to favorites
local function saveToFavorites(note)
  local file = File(FAVORITES_PATH)
  if not file.exists() then
    file.getParentFile().mkdirs()
    file.createNewFile()
  end

  local fileWriter = FileWriter(file, true)
  local bufferedWriter = BufferedWriter(fileWriter)

  bufferedWriter.write(note)
  bufferedWriter.newLine()

  bufferedWriter.close()
  fileWriter.close()

  Toast.makeText(activity, "Note added to Favorites", Toast.LENGTH_SHORT).show()
end

-- Load favorite notes
local function loadFavorites()
  local file = File(FAVORITES_PATH)

  if file.exists() then
    local reader = BufferedReader(FileReader(file))
    local line

    repeat
      line = reader.readLine()
      if line then
        notesList.add(line)
      end
    until not line

    reader.close()
    notesAdapter.notifyDataSetChanged()
  end
end

local mediaRecorder = MediaRecorder()
local isRecording = false
local recordingFile

-- Function to start recording
local function startRecording()
  isRecording = true

  -- Set the output file
  local fileName = os.date("%Y%m%d_%H%M%S") .. ".3gp"
  recordingFile = File(RECORDING_FOLDER_PATH, fileName)

  -- Prepare the media recorder
  mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC)
  mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP)
  mediaRecorder.setOutputFile(recordingFile.getAbsolutePath())
  mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB)
  
  -- Create the directory if it does not exist
  recordingFile.getParentFile().mkdirs()

  -- Start recording
  mediaRecorder.prepare()
  mediaRecorder.start()
end

-- Function to stop recording
local function stopRecording()
  isRecording = false
  mediaRecorder.stop()
  mediaRecorder.reset()
  mediaRecorder.release()
end

-- Function to save the recording to local storage
local function saveRecording()
  local nameInputDialog = AlertDialog.Builder(activity)
  local nameInputEditText = EditText(activity)
  nameInputDialog.setView(nameInputEditText)

  nameInputDialog.setPositiveButton("Save", DialogInterface.OnClickListener {
    onClick = function(_, _)
      local recordingName = nameInputEditText.getText().toString()

      if recordingName ~= "" then
        local newRecordingFile = File(RECORDING_FOLDER_PATH .. "/", recordingName .. ".3gp")
        recordingFile.renameTo(newRecordingFile)

        Toast.makeText(activity, "Recording saved", Toast.LENGTH_SHORT).show()
      else
        Toast.makeText(activity, "Please enter a name for the recording", Toast.LENGTH_SHORT).show()
      end
    end
  })

  nameInputDialog.setNegativeButton("Cancel", DialogInterface.OnClickListener {
    onClick = function(_, _)
      recordingFile.delete()
      Toast.makeText(activity, "Recording deleted", Toast.LENGTH_SHORT).show()
    end
  })

  nameInputDialog.show()
end

-- Show pop-up for adding a note
local function showAddNotePopup(existingNote, position)
  local popupLayout = LinearLayout(activity)
  popupLayout.setOrientation(LinearLayout.VERTICAL)

  local dynamicEditText = EditText(activity)
  if existingNote then
    dynamicEditText.setText(existingNote)
  end
  popupLayout.addView(dynamicEditText)

  local addToFavoritesCheckbox = CheckBox(activity)
  addToFavoritesCheckbox.setText("Add to Favorites")
  popupLayout.addView(addToFavoritesCheckbox)

  local addToNotesCheckbox = CheckBox(activity)
  addToNotesCheckbox.setText("Add to Notes")
  popupLayout.addView(addToNotesCheckbox)

  local dynamicCancelButton = Button(activity)
  dynamicCancelButton.setText("Cancel")
  popupLayout.addView(dynamicCancelButton)

  local dynamicSaveButton = Button(activity)
  dynamicSaveButton.setText("Save")
  popupLayout.addView(dynamicSaveButton)

  local popupWindow = PopupWindow(popupLayout, LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT, true)
  popupWindow.showAtLocation(layout, Gravity.CENTER, 0, 0)

  -- Cancel button click listener
  dynamicCancelButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
      popupWindow.dismiss()
    end
  })

  -- Save button click listener
  dynamicSaveButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
      local noteText = dynamicEditText.getText().toString()
      local addToFavorites = addToFavoritesCheckbox.isChecked()
      local addToNotes = addToNotesCheckbox.isChecked()

      if noteText ~= "" then
        if addToFavorites then
          saveToFavorites(noteText)
        end

        if addToNotes then
          if existingNote and position then
            -- Edit existing note
            notesList.set(position, noteText)
            local file = File(STORAGE_PATH)
            local tempFile = File("/storage/emulated/0/解说/Tools/notetaking-tool_pro/temp_notes.txt")

            local scanner = BufferedReader(FileReader(file))
            local writer = BufferedWriter(FileWriter(tempFile))

            local currentLine = 0
            local line

            repeat
              line = scanner.readLine()
              if line then
                if currentLine == position then
                  writer.write(noteText)
                else
                  writer.write(line)
                end
                writer.newLine()
              end
              currentLine = currentLine + 1
            until not line

            scanner.close()
            writer.close()

            -- Delete the original file
            file.delete()

            -- Rename the temporary file to the original file name
            tempFile.renameTo(file)

          else
            -- Add a new note
            saveNote(noteText)
          end
        end

        popupWindow.dismiss()
      else
        Toast.makeText(activity, "Note cannot be empty", Toast.LENGTH_SHORT).show()
      end
    end
  })
end

-- Set button click listeners
searchButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    local searchDialog = AlertDialog.Builder(activity)
    local searchEditText = EditText(activity)
    searchDialog.setView(searchEditText)

    searchDialog.setPositiveButton("Search", DialogInterface.OnClickListener {
      onClick = function(_, _)
        local searchTerm = searchEditText.getText().toString()
        if searchTerm ~= "" then
          notesList.clear()
          local file = File(STORAGE_PATH)
          if file.exists() then
            local reader = BufferedReader(FileReader(file))
            local line

            repeat
              line = reader.readLine()
              if line and line:lower():find(searchTerm:lower()) then
                notesList.add(line)
              end
            until not line

            reader.close()
            notesAdapter.notifyDataSetChanged()
          end
        else
          Toast.makeText(activity, "Please enter a search term", Toast.LENGTH_SHORT).show()
        end
      end
    })

    searchDialog.setNegativeButton("Cancel", nil)

    searchDialog.show()
  end
})

notesButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    notesList.clear()
    loadNotes()
    notesAdapter.notifyDataSetChanged()
  end
})

favoritesButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    notesList.clear()
    loadFavorites()
    notesAdapter.notifyDataSetChanged()
  end
})

addNoteButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    showAddNotePopup()
  end
})

newRecordingButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    local newRecordingLayout = LinearLayout(activity)
    newRecordingLayout.setOrientation(LinearLayout.VERTICAL)

    local recordingNameEditText = EditText(activity)
    recordingNameEditText.setHint("Recording Name")
    newRecordingLayout.addView(recordingNameEditText)

    local recordButton = Button(activity)
    recordButton.setText("Record")
    newRecordingLayout.addView(recordButton)

    local stopButton = Button(activity)
    stopButton.setText("Stop")
    stopButton.setEnabled(false)
    newRecordingLayout.addView(stopButton)

    local saveButton = Button(activity)
    saveButton.setText("Save to Local Storage")
    saveButton.setEnabled(false)
    newRecordingLayout.addView(saveButton)

    local deleteButton = Button(activity)
    deleteButton.setText("Delete Recording")
    deleteButton.setEnabled(false)
    newRecordingLayout.addView(deleteButton)

    local newRecordingPopup = PopupWindow(newRecordingLayout, LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT, true)
    newRecordingPopup.showAtLocation(layout, Gravity.CENTER, 0, 0)

    local isRecording = false

    recordButton.setOnClickListener(View.OnClickListener {
      onClick = function(_)
        local recordingName = recordingNameEditText.getText().toString()
        if recordingName ~= "" then
          isRecording = true
          recordButton.setEnabled(false)
          stopButton.setEnabled(true)

          -- Start recording
          startRecording()
        else
          Toast.makeText(activity, "Please enter a name for the recording", Toast.LENGTH_SHORT).show()
        end
      end
    })

    stopButton.setOnClickListener(View.OnClickListener {
      onClick = function(_)
        isRecording = false
        recordButton.setEnabled(true)
        saveButton.setEnabled(true)
        deleteButton.setEnabled(true)

        -- Stop recording
        stopRecording()

        Toast.makeText(activity, "Recording stopped", Toast.LENGTH_SHORT).show()
      end
    })

    saveButton.setOnClickListener(View.OnClickListener {
      onClick = function(_)
        local recordingName = recordingNameEditText.getText().toString()
        if recordingName ~= "" then
          saveRecording()
        else
          Toast.makeText(activity, "Please enter a name for the recording", Toast.LENGTH_SHORT).show()
        end
      end
    })

    deleteButton.setOnClickListener(View.OnClickListener {
      onClick = function(_)
        recordingFile.delete()
        Toast.makeText(activity, "Recording deleted", Toast.LENGTH_SHORT).show()
      end
    })
  end
})

developerInfoButton.setOnClickListener(View.OnClickListener {
  onClick = function(_)
    local developerInfoDialog = AlertDialog.Builder(activity)
    developerInfoDialog.setTitle("Developer Info")
    developerInfoDialog.setMessage("Hello there! 👋\n\nI'm the developer of this note-taking app.\n" .. "My name is [INSERT YOUR NAME HERE]. I'm passionate about creating useful tools like this one!\n\n" .. "I'm currently working on making the app even better for you. If you have any feedback or suggestions,\n" .. "feel free to reach out. Your input is invaluable!\n\n" .. "Thank you very much for choosing our tool and thank you very much for using it.\n" .. "Happy Noting!")
    developerInfoDialog.setPositiveButton("OK", nil)
    developerInfoDialog.show()
  end
})

-- Add UI elements to layout
layout.addView(headingTextView)
layout.addView(searchButton)
layout.addView(notesButton)
layout.addView(favoritesButton)
layout.addView(addNoteButton)
layout.addView(newRecordingButton)
layout.addView(developerInfoButton)
layout.addView(notesListView)

-- Load existing notes when the app starts
loadNotes()

-- Context menu for notes (Edit, Delete, Add to Favorites)
notesListView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener {
  onItemLongClick = function(_, _, position, _)
    local selectedNote = notesList.get(position)
    local popupMenu = PopupMenu(activity, notesListView.getChildAt(position))
    popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 1, "Edit")
    popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 2, "Delete")
    popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 3, "Add to Favorites")

    popupMenu.setOnMenuItemClickListener(PopupMenu.OnMenuItemClickListener {
      onMenuItemClick = function(item)
        if item.getTitle() == "Edit" then
          showAddNotePopup(selectedNote, position)
        elseif item.getTitle() == "Delete" then
          removeNote(position)
        elseif item.getTitle() == "Add to Favorites" then
          saveToFavorites(selectedNote)
        end
        return true
      end
    })

    popupMenu.show()
    return true
  end
})

-- Set the layout as the content view
activity.setContentView(layout)